# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.



# The name this table is to be known by, with the format of the mappings in
# the main body of the table, and what all code points missing from this file
# map to.
$Unicode::UCD::SwashInfo{'ToBc'}{'format'} = 's'; # string
$Unicode::UCD::SwashInfo{'ToBc'}{'missing'} = 'L';

return <<'END';
0	8	BN
9		S
A		B
B		S
C		WS
D		B
E	1B	BN
1C	1E	B
1F		S
20		WS
21	22	ON
23	25	ET
26	2A	ON
2B		ES
2C		CS
2D		ES
2E	2F	CS
30	39	EN
3A		CS
3B	40	ON
5B	60	ON
7B	7E	ON
7F	84	BN
85		B
86	9F	BN
A0		CS
A1		ON
A2	A5	ET
A6	A9	ON
AB	AC	ON
AD		BN
AE	AF	ON
B0	B1	ET
B2	B3	EN
B4		ON
B6	B8	ON
B9		EN
BB	BF	ON
D7		ON
F7		ON
2B9	2BA	ON
2C2	2CF	ON
2D2	2DF	ON
2E5	2ED	ON
2EF	2FF	ON
300	36F	NSM
374	375	ON
37E		ON
384	385	ON
387		ON
3F6		ON
483	489	NSM
58A		ON
58D	58E	ON
58F		ET
590		R
591	5BD	NSM
5BE		R
5BF		NSM
5C0		R
5C1	5C2	NSM
5C3		R
5C4	5C5	NSM
5C6		R
5C7		NSM
5C8	5FF	R
600	605	AN
606	607	ON
608		AL
609	60A	ET
60B		AL
60C		CS
60D		AL
60E	60F	ON
610	61A	NSM
61B	64A	AL
64B	65F	NSM
660	669	AN
66A		ET
66B	66C	AN
66D	66F	AL
670		NSM
671	6D5	AL
6D6	6DC	NSM
6DD		AN
6DE		ON
6DF	6E4	NSM
6E5	6E6	AL
6E7	6E8	NSM
6E9		ON
6EA	6ED	NSM
6EE	6EF	AL
6F0	6F9	EN
6FA	710	AL
711		NSM
712	72F	AL
730	74A	NSM
74B	7A5	AL
7A6	7B0	NSM
7B1	7BF	AL
7C0	7EA	R
7EB	7F3	NSM
7F4	7F5	R
7F6	7F9	ON
7FA	7FC	R
7FD		NSM
7FE	815	R
816	819	NSM
81A		R
81B	823	NSM
824		R
825	827	NSM
828		R
829	82D	NSM
82E	858	R
859	85B	NSM
85C	85F	R
860	88F	AL
890	891	AN
892	896	AL
897	89F	NSM
8A0	8C9	AL
8CA	8E1	NSM
8E2		AN
8E3	902	NSM
93A		NSM
93C		NSM
941	948	NSM
94D		NSM
951	957	NSM
962	963	NSM
981		NSM
9BC		NSM
9C1	9C4	NSM
9CD		NSM
9E2	9E3	NSM
9F2	9F3	ET
9FB		ET
9FE		NSM
A01	A02	NSM
A3C		NSM
A41	A42	NSM
A47	A48	NSM
A4B	A4D	NSM
A51		NSM
A70	A71	NSM
A75		NSM
A81	A82	NSM
ABC		NSM
AC1	AC5	NSM
AC7	AC8	NSM
ACD		NSM
AE2	AE3	NSM
AF1		ET
AFA	AFF	NSM
B01		NSM
B3C		NSM
B3F		NSM
B41	B44	NSM
B4D		NSM
B55	B56	NSM
B62	B63	NSM
B82		NSM
BC0		NSM
BCD		NSM
BF3	BF8	ON
BF9		ET
BFA		ON
C00		NSM
C04		NSM
C3C		NSM
C3E	C40	NSM
C46	C48	NSM
C4A	C4D	NSM
C55	C56	NSM
C62	C63	NSM
C78	C7E	ON
C81		NSM
CBC		NSM
CCC	CCD	NSM
CE2	CE3	NSM
D00	D01	NSM
D3B	D3C	NSM
D41	D44	NSM
D4D		NSM
D62	D63	NSM
D81		NSM
DCA		NSM
DD2	DD4	NSM
DD6		NSM
E31		NSM
E34	E3A	NSM
E3F		ET
E47	E4E	NSM
EB1		NSM
EB4	EBC	NSM
EC8	ECE	NSM
F18	F19	NSM
F35		NSM
F37		NSM
F39		NSM
F3A	F3D	ON
F71	F7E	NSM
F80	F84	NSM
F86	F87	NSM
F8D	F97	NSM
F99	FBC	NSM
FC6		NSM
102D	1030	NSM
1032	1037	NSM
1039	103A	NSM
103D	103E	NSM
1058	1059	NSM
105E	1060	NSM
1071	1074	NSM
1082		NSM
1085	1086	NSM
108D		NSM
109D		NSM
135D	135F	NSM
1390	1399	ON
1400		ON
1680		WS
169B	169C	ON
1712	1714	NSM
1732	1733	NSM
1752	1753	NSM
1772	1773	NSM
17B4	17B5	NSM
17B7	17BD	NSM
17C6		NSM
17C9	17D3	NSM
17DB		ET
17DD		NSM
17F0	17F9	ON
1800	180A	ON
180B	180D	NSM
180E		BN
180F		NSM
1885	1886	NSM
18A9		NSM
1920	1922	NSM
1927	1928	NSM
1932		NSM
1939	193B	NSM
1940		ON
1944	1945	ON
19DE	19FF	ON
1A17	1A18	NSM
1A1B		NSM
1A56		NSM
1A58	1A5E	NSM
1A60		NSM
1A62		NSM
1A65	1A6C	NSM
1A73	1A7C	NSM
1A7F		NSM
1AB0	1ACE	NSM
1B00	1B03	NSM
1B34		NSM
1B36	1B3A	NSM
1B3C		NSM
1B42		NSM
1B6B	1B73	NSM
1B80	1B81	NSM
1BA2	1BA5	NSM
1BA8	1BA9	NSM
1BAB	1BAD	NSM
1BE6		NSM
1BE8	1BE9	NSM
1BED		NSM
1BEF	1BF1	NSM
1C2C	1C33	NSM
1C36	1C37	NSM
1CD0	1CD2	NSM
1CD4	1CE0	NSM
1CE2	1CE8	NSM
1CED		NSM
1CF4		NSM
1CF8	1CF9	NSM
1DC0	1DFF	NSM
1FBD		ON
1FBF	1FC1	ON
1FCD	1FCF	ON
1FDD	1FDF	ON
1FED	1FEF	ON
1FFD	1FFE	ON
2000	200A	WS
200B	200D	BN
200F		R
2010	2027	ON
2028		WS
2029		B
202A		LRE
202B		RLE
202C		PDF
202D		LRO
202E		RLO
202F		CS
2030	2034	ET
2035	2043	ON
2044		CS
2045	205E	ON
205F		WS
2060	2065	BN
2066		LRI
2067		RLI
2068		FSI
2069		PDI
206A	206F	BN
2070		EN
2074	2079	EN
207A	207B	ES
207C	207E	ON
2080	2089	EN
208A	208B	ES
208C	208E	ON
20A0	20CF	ET
20D0	20F0	NSM
2100	2101	ON
2103	2106	ON
2108	2109	ON
2114		ON
2116	2118	ON
211E	2123	ON
2125		ON
2127		ON
2129		ON
212E		ET
213A	213B	ON
2140	2144	ON
214A	214D	ON
2150	215F	ON
2189	218B	ON
2190	2211	ON
2212		ES
2213		ET
2214	2335	ON
237B	2394	ON
2396	2429	ON
2440	244A	ON
2460	2487	ON
2488	249B	EN
24EA	26AB	ON
26AD	27FF	ON
2900	2B73	ON
2B76	2B95	ON
2B97	2BFF	ON
2CE5	2CEA	ON
2CEF	2CF1	NSM
2CF9	2CFF	ON
2D7F		NSM
2DE0	2DFF	NSM
2E00	2E5D	ON
2E80	2E99	ON
2E9B	2EF3	ON
2F00	2FD5	ON
2FF0	2FFF	ON
3000		WS
3001	3004	ON
3008	3020	ON
302A	302D	NSM
3030		ON
3036	3037	ON
303D	303F	ON
3099	309A	NSM
309B	309C	ON
30A0		ON
30FB		ON
31C0	31E5	ON
31EF		ON
321D	321E	ON
3250	325F	ON
327C	327E	ON
32B1	32BF	ON
32CC	32CF	ON
3377	337A	ON
33DE	33DF	ON
33FF		ON
4DC0	4DFF	ON
A490	A4C6	ON
A60D	A60F	ON
A66F	A672	NSM
A673		ON
A674	A67D	NSM
A67E	A67F	ON
A69E	A69F	NSM
A6F0	A6F1	NSM
A700	A721	ON
A788		ON
A802		NSM
A806		NSM
A80B		NSM
A825	A826	NSM
A828	A82B	ON
A82C		NSM
A838	A839	ET
A874	A877	ON
A8C4	A8C5	NSM
A8E0	A8F1	NSM
A8FF		NSM
A926	A92D	NSM
A947	A951	NSM
A980	A982	NSM
A9B3		NSM
A9B6	A9B9	NSM
A9BC	A9BD	NSM
A9E5		NSM
AA29	AA2E	NSM
AA31	AA32	NSM
AA35	AA36	NSM
AA43		NSM
AA4C		NSM
AA7C		NSM
AAB0		NSM
AAB2	AAB4	NSM
AAB7	AAB8	NSM
AABE	AABF	NSM
AAC1		NSM
AAEC	AAED	NSM
AAF6		NSM
AB6A	AB6B	ON
ABE5		NSM
ABE8		NSM
ABED		NSM
FB1D		R
FB1E		NSM
FB1F	FB28	R
FB29		ES
FB2A	FB4F	R
FB50	FD3D	AL
FD3E	FD4F	ON
FD50	FDCE	AL
FDCF		ON
FDD0	FDEF	BN
FDF0	FDFC	AL
FDFD	FDFF	ON
FE00	FE0F	NSM
FE10	FE19	ON
FE20	FE2F	NSM
FE30	FE4F	ON
FE50		CS
FE51		ON
FE52		CS
FE54		ON
FE55		CS
FE56	FE5E	ON
FE5F		ET
FE60	FE61	ON
FE62	FE63	ES
FE64	FE66	ON
FE68		ON
FE69	FE6A	ET
FE6B		ON
FE70	FEFE	AL
FEFF		BN
FF01	FF02	ON
FF03	FF05	ET
FF06	FF0A	ON
FF0B		ES
FF0C		CS
FF0D		ES
FF0E	FF0F	CS
FF10	FF19	EN
FF1A		CS
FF1B	FF20	ON
FF3B	FF40	ON
FF5B	FF65	ON
FFE0	FFE1	ET
FFE2	FFE4	ON
FFE5	FFE6	ET
FFE8	FFEE	ON
FFF0	FFF8	BN
FFF9	FFFD	ON
FFFE	FFFF	BN
10101		ON
10140	1018C	ON
10190	1019C	ON
101A0		ON
101FD		NSM
102E0		NSM
102E1	102FB	EN
10376	1037A	NSM
10800	1091E	R
1091F		ON
10920	10A00	R
10A01	10A03	NSM
10A04		R
10A05	10A06	NSM
10A07	10A0B	R
10A0C	10A0F	NSM
10A10	10A37	R
10A38	10A3A	NSM
10A3B	10A3E	R
10A3F		NSM
10A40	10AE4	R
10AE5	10AE6	NSM
10AE7	10B38	R
10B39	10B3F	ON
10B40	10CFF	R
10D00	10D23	AL
10D24	10D27	NSM
10D28	10D2F	AL
10D30	10D39	AN
10D3A	10D3F	AL
10D40	10D49	AN
10D4A	10D68	R
10D69	10D6D	NSM
10D6E		ON
10D6F	10E5F	R
10E60	10E7E	AN
10E7F	10EAA	R
10EAB	10EAC	NSM
10EAD	10EBF	R
10EC0	10EFB	AL
10EFC	10EFF	NSM
10F00	10F2F	R
10F30	10F45	AL
10F46	10F50	NSM
10F51	10F6F	AL
10F70	10F81	R
10F82	10F85	NSM
10F86	10FFF	R
11001		NSM
11038	11046	NSM
11052	11065	ON
11070		NSM
11073	11074	NSM
1107F	11081	NSM
110B3	110B6	NSM
110B9	110BA	NSM
110C2		NSM
11100	11102	NSM
11127	1112B	NSM
1112D	11134	NSM
11173		NSM
11180	11181	NSM
111B6	111BE	NSM
111C9	111CC	NSM
111CF		NSM
1122F	11231	NSM
11234		NSM
11236	11237	NSM
1123E		NSM
11241		NSM
112DF		NSM
112E3	112EA	NSM
11300	11301	NSM
1133B	1133C	NSM
11340		NSM
11366	1136C	NSM
11370	11374	NSM
113BB	113C0	NSM
113CE		NSM
113D0		NSM
113D2		NSM
113E1	113E2	NSM
11438	1143F	NSM
11442	11444	NSM
11446		NSM
1145E		NSM
114B3	114B8	NSM
114BA		NSM
114BF	114C0	NSM
114C2	114C3	NSM
115B2	115B5	NSM
115BC	115BD	NSM
115BF	115C0	NSM
115DC	115DD	NSM
11633	1163A	NSM
1163D		NSM
1163F	11640	NSM
11660	1166C	ON
116AB		NSM
116AD		NSM
116B0	116B5	NSM
116B7		NSM
1171D		NSM
1171F		NSM
11722	11725	NSM
11727	1172B	NSM
1182F	11837	NSM
11839	1183A	NSM
1193B	1193C	NSM
1193E		NSM
11943		NSM
119D4	119D7	NSM
119DA	119DB	NSM
119E0		NSM
11A01	11A06	NSM
11A09	11A0A	NSM
11A33	11A38	NSM
11A3B	11A3E	NSM
11A47		NSM
11A51	11A56	NSM
11A59	11A5B	NSM
11A8A	11A96	NSM
11A98	11A99	NSM
11C30	11C36	NSM
11C38	11C3D	NSM
11C92	11CA7	NSM
11CAA	11CB0	NSM
11CB2	11CB3	NSM
11CB5	11CB6	NSM
11D31	11D36	NSM
11D3A		NSM
11D3C	11D3D	NSM
11D3F	11D45	NSM
11D47		NSM
11D90	11D91	NSM
11D95		NSM
11D97		NSM
11EF3	11EF4	NSM
11F00	11F01	NSM
11F36	11F3A	NSM
11F40		NSM
11F42		NSM
11F5A		NSM
11FD5	11FDC	ON
11FDD	11FE0	ET
11FE1	11FF1	ON
13440		NSM
13447	13455	NSM
1611E	16129	NSM
1612D	1612F	NSM
16AF0	16AF4	NSM
16B30	16B36	NSM
16F4F		NSM
16F8F	16F92	NSM
16FE2		ON
16FE4		NSM
1BC9D	1BC9E	NSM
1BCA0	1BCA3	BN
1CC00	1CCD5	ON
1CCF0	1CCF9	EN
1CD00	1CEB3	ON
1CF00	1CF2D	NSM
1CF30	1CF46	NSM
1D167	1D169	NSM
1D173	1D17A	BN
1D17B	1D182	NSM
1D185	1D18B	NSM
1D1AA	1D1AD	NSM
1D1E9	1D1EA	ON
1D200	1D241	ON
1D242	1D244	NSM
1D245		ON
1D300	1D356	ON
1D6C1		ON
1D6DB		ON
1D6FB		ON
1D715		ON
1D735		ON
1D74F		ON
1D76F		ON
1D789		ON
1D7A9		ON
1D7C3		ON
1D7CE	1D7FF	EN
1DA00	1DA36	NSM
1DA3B	1DA6C	NSM
1DA75		NSM
1DA84		NSM
1DA9B	1DA9F	NSM
1DAA1	1DAAF	NSM
1E000	1E006	NSM
1E008	1E018	NSM
1E01B	1E021	NSM
1E023	1E024	NSM
1E026	1E02A	NSM
1E08F		NSM
1E130	1E136	NSM
1E2AE		NSM
1E2EC	1E2EF	NSM
1E2FF		ET
1E4EC	1E4EF	NSM
1E5EE	1E5EF	NSM
1E800	1E8CF	R
1E8D0	1E8D6	NSM
1E8D7	1E943	R
1E944	1E94A	NSM
1E94B	1EC6F	R
1EC70	1ECBF	AL
1ECC0	1ECFF	R
1ED00	1ED4F	AL
1ED50	1EDFF	R
1EE00	1EEEF	AL
1EEF0	1EEF1	ON
1EEF2	1EEFF	AL
1EF00	1EFFF	R
1F000	1F02B	ON
1F030	1F093	ON
1F0A0	1F0AE	ON
1F0B1	1F0BF	ON
1F0C1	1F0CF	ON
1F0D1	1F0F5	ON
1F100	1F10A	EN
1F10B	1F10F	ON
1F12F		ON
1F16A	1F16F	ON
1F1AD		ON
1F260	1F265	ON
1F300	1F6D7	ON
1F6DC	1F6EC	ON
1F6F0	1F6FC	ON
1F700	1F776	ON
1F77B	1F7D9	ON
1F7E0	1F7EB	ON
1F7F0		ON
1F800	1F80B	ON
1F810	1F847	ON
1F850	1F859	ON
1F860	1F887	ON
1F890	1F8AD	ON
1F8B0	1F8BB	ON
1F8C0	1F8C1	ON
1F900	1FA53	ON
1FA60	1FA6D	ON
1FA70	1FA7C	ON
1FA80	1FA89	ON
1FA8F	1FAC6	ON
1FACE	1FADC	ON
1FADF	1FAE9	ON
1FAF0	1FAF8	ON
1FB00	1FB92	ON
1FB94	1FBEF	ON
1FBF0	1FBF9	EN
1FFFE	1FFFF	BN
2FFFE	2FFFF	BN
3FFFE	3FFFF	BN
4FFFE	4FFFF	BN
5FFFE	5FFFF	BN
6FFFE	6FFFF	BN
7FFFE	7FFFF	BN
8FFFE	8FFFF	BN
9FFFE	9FFFF	BN
AFFFE	AFFFF	BN
BFFFE	BFFFF	BN
CFFFE	CFFFF	BN
DFFFE	E00FF	BN
E0100	E01EF	NSM
E01F0	E0FFF	BN
EFFFE	EFFFF	BN
FFFFE	FFFFF	BN
10FFFE	10FFFF	BN
END
